
package com.example.viewoverlay;

import android.app.Activity;
import android.graphics.Point;
import android.graphics.Rect;
import android.graphics.drawable.Drawable;
import android.os.Bundle;
import android.view.MotionEvent;
import android.view.View;

import java.util.ArrayList;

public class MainActivity extends Activity implements View.OnTouchListener {

    private ArrayList<Drawable> mMarkers;
    private Drawable mTrackingMarker;
    private Point mTrackingPoint;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);

        //给整个界面设置触摸监听
        findViewById(R.id.textview).setOnTouchListener(this);

        mMarkers = new ArrayList<Drawable>();
    }

    /*
     * 触摸事件处理
     */
    @Override
    public boolean onTouch(View v, MotionEvent event) {
        int x = (int) event.getX();
        int y = (int) event.getY();
        switch (event.getAction()) {
            case MotionEvent.ACTION_DOWN:
                Drawable current = markerAt(x, y);
                if (current == null) {
                    mTrackingMarker = addBox(v, x, y);
                    mTrackingPoint = new Point(x, y);
                } else {
                    //移除前面的方框
                    removeMarker(v, current);
                }
                break;
            case MotionEvent.ACTION_MOVE:
                if (mTrackingMarker != null) {
                    resizeBox(v, mTrackingMarker, mTrackingPoint, x, y);
                }
                break;
            case MotionEvent.ACTION_UP:
            case MotionEvent.ACTION_CANCEL:
                mTrackingMarker = null;
                mTrackingPoint = null;
                break;
        }
        return true;
    }


    /*
     * 按照指定的坐标添加一个方框
     */
    private Drawable addBox(View v, int x, int y) {
        Drawable box = getResources().getDrawable(R.drawable.box);

        //以触摸点为开始点
        Rect bounds = new Rect(x, y, x, y);
        box.setBounds(bounds);

        //添加到ViewOverlay
        mMarkers.add(box);
        v.getOverlay().add(box);

        return box;
    }

    /*
     * 更新已经存在的方框
     */
    private void resizeBox(View v, Drawable target, Point trackingPoint, int x, int y) {
        Rect bounds = new Rect(target.getBounds());
        //如果触摸点在已经存在的点的左边更新左边位置
        //else说明在右边，更新右边的位置
        if (x < trackingPoint.x) {
            bounds.left = x;
        } else {
            bounds.right = x;
        }

        //如果新的触摸点在已经存在点的上边，更新上边的位置
        //else更新下边的位置
        if (y < trackingPoint.y) {
            bounds.top = y;
        } else {
            bounds.bottom = y;
        }

        //更新方框的边框
        target.setBounds(bounds);
        v.invalidate();
    }


    /*
     * 移除marker
     */
    private void removeMarker(View v, Drawable marker) {
        mMarkers.remove(marker);
        v.getOverlay().remove(marker);
    }

    /*
     * 按坐标找到第一个marker
     */
    private Drawable markerAt(int x, int y) {
        for (Drawable marker : mMarkers) {
            if (marker.getBounds().contains(x, y)) {
                return marker;
            }
        }

        return null;
    }
}
